@@ -9,18 +9,35 @@ function-to-be-minimized (residual function) in terms of these Parameters.
 
    Copyright (c) 2011 Matthew Newville, The University of Chicago
    <newville@cars.uchicago.edu>
-
 """
 
 from numpy import sqrt
-from .asteval import Interpreter, NameFinder
-from .astutils import valid_symbol_name
 
 from scipy.optimize import leastsq as scipy_leastsq
-from scipy.optimize import anneal as scipy_anneal
-from scipy.optimize import fmin_l_bfgs_b as scipy_lbfgsb
+#from scipy.optimize import anneal as scipy_anneal
+#from scipy.optimize import fmin_l_bfgs_b as scipy_lbfgsb
 
-class Parameters(dict):
+try:
+    from collections import OrderedDict
+except ImportError:
+    from ordereddict import OrderedDict
+
+import re
+NAME_MATCH = re.compile(r"[a-z_][a-z0-9_]*$").match
+
+def _issymbol_name(name):
+    "input is a valid name"
+    lname = name[:].lower()
+    return NAME_MATCH(lname) is not None
+
+try:
+    from asteval import Interpreter, NameFinder, valid_symbol_name
+    HAS_ASTEVAL = True
+except:
+    HAS_ASTEVAL = False
+    valid_symbol_name = _issymbol_name
+
+class Parameters(OrderedDict):
     """a custom dictionary of Parameters.  All keys must be
     strings, and valid Python symbol names, and all values
     must be Parameters.
@@ -32,7 +49,7 @@ class Parameters(dict):
     add_many()
     """
     def __init__(self, *args, **kwds):
-        dict.__init__(self)
+        OrderedDict.__init__(self)
         self.update(*args, **kwds)
 
     def __setitem__(self, key, value):
@@ -41,7 +58,7 @@ class Parameters(dict):
                 raise KeyError("'%s' is not a valid Parameters name" % key)
         if value is not None and not isinstance(value, Parameter):
             raise ValueError("'%s' is not a Parameter" % value)
-        dict.__setitem__(self, key, value)
+        OrderedDict.__setitem__(self, key, value)
         value.name = key
 
     def add(self, name, value=None, vary=True, expr=None,
@@ -66,9 +83,9 @@ class Parameters(dict):
                     (name3, val3, False, None, None, None))
 
         """
-        for name, value, vary, min, max, expr in parlist:
+        for name, value, vary, vmin, vmax, expr in parlist:
             self.add(name, value=value, vary=vary,
-                     min=min, max=max, expr=expr)
+                     min=vmin, max=vmax, expr=expr)
 
 class Parameter(object):
     """A Parameter is the basic Parameter going
@@ -76,16 +93,19 @@ class Parameter(object):
     value, vary, max_value, min_value, constraint expression.
     The value and min/max values will be be set to floats.
     """
-    def __init__(self, value=None, vary=True, name=None,
+    def __init__(self, name=None, value=None, vary=True,
                  min=None, max=None, expr=None, **kws):
+        self.name = name
         self.value = value
+        self.init_value = value
         self.min = min
         self.max = max
         self.vary = vary
         self.expr = expr
-        self.name = name
         self.stderr = None
         self.correl = None
+        if not HAS_ASTEVAL and expr is not None:
+            raise Warning("cannot use constraint expressions -- asteval not installed!")
 
     def __repr__(self):
         s = []
@@ -100,7 +120,6 @@ class Parameter(object):
         s.append("bounds=[%s:%s]" % (repr(self.min), repr(self.max)))
         if self.expr is not None:
             s.append("expr='%s'" % (self.expr))
-
         return "<Parameter %s>" % ', '.join(s)
 
 class MinimizerException(Exception):
@@ -123,15 +142,16 @@ def check_ast_errors(error):
 
 class Minimizer(object):
     """general minimizer"""
-    err_nonparam = "params must be a minimizer.Parameters() instance"
-    err_maxfev   = """Too many function calls (max set to  %%i)!  Use:
+    err_nonparam = \
+     "params must be a minimizer.Parameters() instance or list of Parameters()"
+    err_maxfev   = """Too many function calls (max set to  %i)!  Use:
     minimize(func, params, ...., maxfev=NNN)
 or set  leastsq_kws['maxfev']  to increase this maximum."""
 
-    def __init__(self, userfcn, params, fcn_args=None, fcn_kws=None, **kws):
+    def __init__(self, userfcn, params, fcn_args=None, fcn_kws=None,
+                 iter_cb=None, scale_covar=True, **kws):
         self.userfcn = userfcn
-        self.params = params
-
+        self.__set_params(params)
         self.userargs = fcn_args
         if self.userargs is None:
             self.userargs = []
@@ -140,9 +160,18 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         if self.userkws is None:
             self.userkws = {}
         self.kws = kws
+        self.iter_cb = iter_cb
+        self.scale_covar = scale_covar
+        self.nfev = 0
+        self.nfree = 0
+        self.message = None
         self.var_map = []
-        self.asteval = Interpreter()
-        self.namefinder = NameFinder()
+        self.jacfcn = None
+        self.asteval = None
+        if HAS_ASTEVAL:
+            self.asteval = Interpreter()
+            self.namefinder = NameFinder()
+        self.__prepared = False
 
     def __update_paramval(self, name):
         """
@@ -159,10 +188,10 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
         par = self.params[name]
         val = par.value
-        if par.expr is not None:
+        if HAS_ASTEVAL and par.expr is not None:
             for dep in par.deps:
                 self.__update_paramval(dep)
-            val = self.asteval.interp(par.ast)
+            val = self.asteval.run(par.ast)
             check_ast_errors(self.asteval.error)
         # apply min/max
         if par.min is not None:
@@ -170,7 +199,8 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         if par.max is not None:
             val = min(val, par.max)
 
-        self.asteval.symtable[name] = par.value = float(val)
+        if HAS_ASTEVAL:
+            self.asteval.symtable[name] = par.value = float(val)
         self.updated[name] = True
 
     def __residual(self, fvars):
@@ -184,31 +214,67 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         # set parameter values
         for varname, val in zip(self.var_map, fvars):
             self.params[varname].value = val
+        self.nfev = self.nfev + 1
 
         self.updated = dict([(name, False) for name in self.params])
         for name in self.params:
             self.__update_paramval(name)
 
-        sout = []
-        for i in self.params.values():
-            sout.append('%s=%.5f'  % (i.name, (i.value)))
-        return self.userfcn(self.params, *self.userargs, **self.userkws)
+        out = self.userfcn(self.params, *self.userargs, **self.userkws)
+        if hasattr(self.iter_cb, '__call__'):
+            self.iter_cb(self.params, self.nfev, out,
+                         *self.userargs, **self.userkws)
+        return out
 
-    def prepare_fit(self):
-        """prepare parameters for fit"""
+    def __jacobian(self, fvars):
+        """
+        analytical jacobian to be used with the Levenberg-Marquardt
 
-        # determine which parameters are actually variables
-        # and which are defined expressions.
-        if not isinstance(self.params, Parameters):
+        modified 02-01-2012 by Glenn Jones, Aberystwyth University
+        """
+        for varname, val in zip(self.var_map, fvars):
+            self.params[varname].value = val
+        self.nfev = self.nfev + 1
+
+        self.updated = dict([(name, False) for name in self.params])
+        for name in self.params:
+            self.__update_paramval(name)
+
+        # computing the jacobian
+        return self.jacfcn(self.params, *self.userargs, **self.userkws)
+
+    def __set_params(self, params):
+        """ set internal self.params from a Parameters object or
+        a list/tuple of Parameters"""
+        if params is None or isinstance(params, Parameters):
+            self.params = params
+        elif isinstance(params, (list, tuple)):
+            _params = Parameters()
+            for _par in params:
+                if not isinstance(_par, Parameter):
+                    raise MinimizerException(self.err_nonparam)
+                else:
+                    _params[_par.name] = _par
+            self.params = _params
+        else:
             raise MinimizerException(self.err_nonparam)
 
+    def prepare_fit(self, params=None):
+        """prepare parameters for fit"""
+        # determine which parameters are actually variables
+        # and which are defined expressions.
+        if params is None and self.params is not None and self.__prepared:
+            return
+        if params is not None and self.params is None:
+            self.__set_params(params)
+        self.nfev = 0
         self.var_map = []
         self.vars = []
         self.vmin = []
         self.vmax = []
         for name, par in self.params.items():
-            if par.expr is not None:
-                par.ast = self.asteval.compile(par.expr)
+            if HAS_ASTEVAL and par.expr is not None:
+                par.ast = self.asteval.parse(par.expr)
                 check_ast_errors(self.asteval.error)
                 par.vary = False
                 par.deps = []
@@ -218,14 +284,14 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
                     if (symname in self.params and
                         symname not in par.deps):
                         par.deps.append(symname)
-
             elif par.vary:
                 self.var_map.append(name)
                 self.vars.append(par.value)
                 self.vmin.append(par.min)
                 self.vmax.append(par.max)
 
-            self.asteval.symtable[name] = par.value
+            if HAS_ASTEVAL:
+                self.asteval.symtable[name] = par.value
             par.init_value = par.value
             if par.name is None:
                 par.name = name
@@ -238,6 +304,7 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.updated = dict([(name, False) for name in self.params])
         for name in self.params:
             self.__update_paramval(name)
+        self.__prepared = True
 
     def anneal(self, schedule='cauchy', **kws):
         """
@@ -249,8 +316,7 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
         self.prepare_fit()
         sakws = dict(full_output=1, schedule=sched,
-                     maxiter = 2000 * (self.nvarys + 1),
-                     upper = self.vmax, lower=self.vmin)
+                     maxiter = 2000 * (self.nvarys + 1))
 
         sakws.update(self.kws)
         sakws.update(kws)
@@ -285,11 +351,11 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.message = info['task']
 
 
-    def leastsq(self, **kws):
+    def leastsq(self, scale_covar=True, **kws):
         """
         use Levenberg-Marquardt minimization to perform fit.
-        This assumes that ModelParameters have been stored into
-        and a function to mi
+        This assumes that ModelParameters have been stored,
+        and a function to minimize has been properly set up.
 
         This wraps scipy.optimize.leastsq, and keyward arguments are passed
         directly as options to scipy.optimize.leastsq
@@ -300,13 +366,17 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         writes outputs to many internal attributes, and
         returns True if fit was successful, False if not.
         """
-
         self.prepare_fit()
         lskws = dict(full_output=1, xtol=1.e-7, ftol=1.e-7,
-                     maxfev= 1000 * (self.nvarys + 1))
+                     gtol=1.e-7, maxfev=1000*(self.nvarys+1), Dfun=None)
+
         lskws.update(self.kws)
         lskws.update(kws)
 
+        if lskws['Dfun'] is not None:
+            self.jacfcn = lskws['Dfun']
+            lskws['Dfun'] = self.__jacobian
+
         lsout = scipy_leastsq(self.__residual, self.vars, **lskws)
         vbest, cov, infodict, errmsg, ier = lsout
 
@@ -343,7 +413,9 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             self.message = '%s. Could not estimate error-bars'
         else:
             self.errorbars = True
-            cov = cov * sum_sqr / self.nfree
+            self.covar = cov
+            if self.scale_covar:
+                cov = cov * sum_sqr / self.nfree
             for ivar, varname in enumerate(self.var_map):
                 par = self.params[varname]
                 par.stderr = sqrt(cov[ivar, ivar])
@@ -355,12 +427,15 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
         return self.success
 
-def minimize(fcn, params, engine='leastsq', args=None, kws=None, **fit_kws):
+def minimize(fcn, params, engine='leastsq', args=None, kws=None,
+             scale_covar=True,
+             iter_cb=None, **fit_kws):
     """simple minimization function,
     finding the values for the params which give the
     minimal sum-of-squares of the array return by fcn
     """
-    fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws, **fit_kws)
+    fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
+                       iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
     if engine == 'anneal':
         fitter.anneal()
     elif engine == 'lbfgsb':
